@@ -27,14 +27,15 @@ from .pyqt import QtCore, QtGui, QtWidgets
 from nexusformat.nexus import (NeXusError, NXgroup, NXfield, NXattr,
                                NXroot, NXentry, NXdata, NXparameters, 
                                NXnote, NXprocess, nxload)
-from .datadialogs import BaseDialog
+from .datadialogs import NXDialog
 from .plotview import NXPlotView
 from .utils import report_error
+from .widgets import NXLineEdit
 
 from ..api.frills.fit import Fit, Function, Parameter
 
     
-class FitDialog(BaseDialog):
+class FitDialog(NXDialog):
     """Dialog to fit one-dimensional NeXus data"""
  
     def __init__(self, entry):
@@ -103,10 +104,10 @@ class FitDialog(BaseDialog):
         plot_label = QtWidgets.QLabel('X-axis:')
         self.plot_min = self.fitview.xaxis.min
         self.plot_max = self.fitview.xaxis.max 
-        self.plot_minbox = QtWidgets.QLineEdit(str(self.plot_min))
+        self.plot_minbox = NXLineEdit(str(self.plot_min))
         self.plot_minbox.setAlignment(QtCore.Qt.AlignRight)
         plot_tolabel = QtWidgets.QLabel(' to ')
-        self.plot_maxbox = QtWidgets.QLineEdit(str(self.plot_max))
+        self.plot_maxbox = NXLineEdit(str(self.plot_max))
         self.plot_maxbox.setAlignment(QtCore.Qt.AlignRight)
         self.plot_checkbox = QtWidgets.QCheckBox('Use Data Points')
         self.plot_checkbox.setVisible(False)
@@ -289,7 +290,7 @@ class FitDialog(BaseDialog):
                                 else:
                                     parameter.vary = False
                             parameters.append(parameter)
-                    f = Function(group, module, parameters, n)
+                    f = Function(group, module, parameters, int(n))
                     self.functions.append(f)
             self.functions = sorted(self.functions)
             for f in self.functions:
@@ -377,12 +378,12 @@ class FitDialog(BaseDialog):
         for p in f.parameters:
             p.parameter_index = row
             p.parameter_box = QtWidgets.QLabel(str(p.parameter_index))
-            p.value_box = QtWidgets.QLineEdit()
+            p.value_box = NXLineEdit()
             p.value_box.setAlignment(QtCore.Qt.AlignRight)
             p.error_box = QtWidgets.QLabel()
-            p.min_box = QtWidgets.QLineEdit('-inf')
+            p.min_box = NXLineEdit('-inf')
             p.min_box.setAlignment(QtCore.Qt.AlignRight)
-            p.max_box = QtWidgets.QLineEdit('inf')
+            p.max_box = NXLineEdit('inf')
             p.max_box.setAlignment(QtCore.Qt.AlignRight)
             p.fixed_box = QtWidgets.QCheckBox()
             self.parameter_grid.addWidget(p.parameter_box, row, 1,
