@@ -18,18 +18,17 @@ from __future__ import (absolute_import, division, print_function,
 import os
 
 from nexusformat.nexus import *
-from .datadialogs import BaseDialog
+from .datadialogs import NXDialog
 
 filetype = "Text File" #Defines the Import Menu label
 
-class BaseImportDialog(BaseDialog):
+class NXImportDialog(NXDialog):
     """Base dialog class for NeXpy import dialogs"""
  
     def __init__(self, parent=None):
 
-        super(BaseImportDialog, self).__init__(parent)
-        from .consoleapp import _mainwindow
-        self.default_directory = _mainwindow.default_directory
+        super(NXImportDialog, self).__init__(parent)
+        self.default_directory = self.mainwindow.default_directory
         self.import_file = None     # must define in subclass
 
     def get_data(self):
@@ -46,11 +45,7 @@ class BaseImportDialog(BaseDialog):
         Completes the data import.
         """
         self.accepted = True
-        from .consoleapp import _mainwindow
-        _mainwindow.import_data()
-        super(BaseImportDialog, self).accept()
-        
-
-def natural_sort(key):
-    import re
-    return [int(t) if t.isdigit() else t for t in re.split(r'(\d+)', key)]    
+        self.mainwindow.import_data()
+        super(NXImportDialog, self).accept()
+ 
+BaseImportDialog = NXImportDialog
