@@ -30,6 +30,7 @@ import re
 import sys
 import webbrowser
 import xml.etree.ElementTree as ET
+from copy import deepcopy
 from threading import Thread
 
 from .pyqt import QtCore, QtGui, QtWidgets, getOpenFileName, getSaveFileName
@@ -108,6 +109,7 @@ class MainWindow(QtWidgets.QMainWindow):
         self.editors = NXScriptWindow(self)
         self.editors.setVisible(False)
         self.log_window = None
+        self._memroot = None
 
         self.console = NXRichJupyterWidget(config=self.config, parent=rightpane)
         self.console.setMinimumSize(700, 100)
@@ -198,6 +200,11 @@ class MainWindow(QtWidgets.QMainWindow):
         from .plotview import plotview
         return plotview
         
+    @property
+    def active_plotview(self):
+        from .plotview import active_plotview
+        return active_plotview
+        
     # Populate the menu bar with common actions and shortcuts
     def add_menu_action(self, menu, action, defer_shortcut=False):
         """Add action to menu as well as self
@@ -260,7 +267,7 @@ class MainWindow(QtWidgets.QMainWindow):
 
         self.openimage_action=QtWidgets.QAction("Open Image...",
             self,
-            shortcut="Ctrl+Alt+O",
+            shortcut=QtGui.QKeySequence("Ctrl+Alt+O"),
             triggered=self.open_image
             )
         self.add_menu_action(self.file_menu, self.openimage_action)
@@ -498,7 +505,7 @@ class MainWindow(QtWidgets.QMainWindow):
 
         self.plot_data_action=QtWidgets.QAction("&Plot Data",
             self,
-            shortcut="Ctrl+P",
+            shortcut=QtGui.QKeySequence("Ctrl+P"),
             triggered=self.plot_data
             )
         self.add_menu_action(self.data_menu, self.plot_data_action)
@@ -510,7 +517,7 @@ class MainWindow(QtWidgets.QMainWindow):
 
         self.overplot_data_action=QtWidgets.QAction("Overplot Data",
             self,
-            shortcut="Ctrl+Alt+P",
+            shortcut=QtGui.QKeySequence("Ctrl+Shift+P"),
             triggered=self.overplot_data
             )
         self.add_menu_action(self.data_menu, self.overplot_data_action)
@@ -542,7 +549,7 @@ class MainWindow(QtWidgets.QMainWindow):
 
         self.view_action=QtWidgets.QAction("View Data",
             self,
-            shortcut="Ctrl+Alt+V",
+            shortcut=QtGui.QKeySequence("Ctrl+Alt+V"),
             triggered=self.view_data
             )
         self.add_menu_action(self.data_menu, self.view_action)
@@ -572,6 +579,13 @@ class MainWindow(QtWidgets.QMainWindow):
             )
         self.add_menu_action(self.data_menu, self.copydata_action)
 
+        self.cutdata_action=QtWidgets.QAction("Cut Data",
+            self,
+            shortcut=QtGui.QKeySequence("Ctrl+Shift+X"),
+            triggered=self.cut_data
+            )
+        self.add_menu_action(self.data_menu, self.cutdata_action)
+
         self.pastedata_action=QtWidgets.QAction("Paste Data",
             self,
             shortcut=QtGui.QKeySequence("Ctrl+Shift+V"),
@@ -588,7 +602,7 @@ class MainWindow(QtWidgets.QMainWindow):
 
         self.delete_action=QtWidgets.QAction("Delete Data",
             self,
-            shortcut=QtGui.QKeySequence("Ctrl+Shift+X"),
+            shortcut=QtGui.QKeySequence("Ctrl+Shift+Alt+X"),
             triggered=self.delete_data
             )
         self.add_menu_action(self.data_menu, self.delete_action)
@@ -660,7 +674,7 @@ class MainWindow(QtWidgets.QMainWindow):
             # disable on OSX, where there is always a menu bar
             self.toggle_menu_bar_act = QtWidgets.QAction("Toggle &Menu Bar",
                 self,
-                shortcut="Ctrl+Shift+M",
+                shortcut=QtGui.QKeySequence("Ctrl+Shift+M"),
                 statusTip="Toggle visibility of menubar",
                 triggered=self.toggle_menu_bar)
             self.add_menu_action(self.view_menu, self.toggle_menu_bar_act)
@@ -691,7 +705,7 @@ class MainWindow(QtWidgets.QMainWindow):
 
         self.reset_font_size = QtWidgets.QAction("Zoom &Reset",
             self,
-            shortcut="Ctrl+0",
+            shortcut=QtGui.QKeySequence("Ctrl+0"),
             triggered=self.reset_font_size_console
             )
         self.add_menu_action(self.view_menu, self.reset_font_size, True)
@@ -764,13 +778,13 @@ class MainWindow(QtWidgets.QMainWindow):
             # add min/maximize actions to OSX, which lacks default bindings.
             self.minimizeAct = QtWidgets.QAction("Mini&mize",
                 self,
-                shortcut="Ctrl+m",
+                shortcut=QtGui.QKeySequence("Ctrl+m"),
                 statusTip="Minimize the window/Restore Normal Size",
                 triggered=self.toggleMinimized)
             # maximize is called 'Zoom' on OSX for some reason
             self.maximizeAct = QtWidgets.QAction("&Zoom",
                 self,
-                shortcut="Ctrl+Shift+M",
+                shortcut=QtGui.QKeySequence("Ctrl+Shift+M"),
                 statusTip="Maximize the window/Restore Normal Size",
                 triggered=self.toggleMaximized)
 
@@ -801,13 +815,6 @@ class MainWindow(QtWidgets.QMainWindow):
             )
         self.add_menu_action(self.window_menu, self.log_action)
 
-        self.panel_action=QtWidgets.QAction("Show Projection Panel",
-            self,
-            shortcut=QtGui.QKeySequence("Ctrl+Shift+P"),
-            triggered=self.show_projection_panel
-            )
-        self.add_menu_action(self.window_menu, self.panel_action)
-
         self.script_window_action=QtWidgets.QAction("Show Script Editor",
             self,
             shortcut=QtGui.QKeySequence("Ctrl+Shift+S"),
@@ -817,30 +824,46 @@ class MainWindow(QtWidgets.QMainWindow):
 
         self.window_menu.addSeparator()
 
-        self.limit_action=QtWidgets.QAction("Change Plot Limits",
+        self.customize_action=QtWidgets.QAction("Show Customize Panel",
             self,
-            shortcut="Ctrl+Alt+L",
-            triggered=self.limit_axes
+            shortcut=QtGui.QKeySequence("Ctrl+Alt+C"),
+            triggered=self.show_customize_panel
+            )
+        self.add_menu_action(self.window_menu, self.customize_action)
+
+        self.limit_action=QtWidgets.QAction("Show Limits Panel",
+            self,
+            shortcut=QtGui.QKeySequence("Ctrl+Alt+L"),
+            triggered=self.show_limits_panel
             )
         self.add_menu_action(self.window_menu, self.limit_action)
 
+        self.panel_action=QtWidgets.QAction("Show Projection Panel",
+            self,
+            shortcut=QtGui.QKeySequence("Ctrl+Alt+P"),
+            triggered=self.show_projection_panel
+            )
+        self.add_menu_action(self.window_menu, self.panel_action)
+
+        self.scan_action=QtWidgets.QAction("Show Scan Panel",
+            self,
+            shortcut=QtGui.QKeySequence("Ctrl+Alt+S"),
+            triggered=self.show_scan_panel
+            )
+        self.add_menu_action(self.window_menu, self.scan_action)
+
+        self.window_menu.addSeparator()
+
         self.reset_limit_action=QtWidgets.QAction("Reset Plot Limits",
             self,
-            shortcut="Ctrl+Alt+Shift+L",
+            shortcut=QtGui.QKeySequence("Ctrl+Alt+Shift+L"),
             triggered=self.reset_axes
             )
         self.add_menu_action(self.window_menu, self.reset_limit_action)
 
-        self.customize_action=QtWidgets.QAction("Customize Plot",
-            self,
-            shortcut="Ctrl+Alt+C",
-            triggered=self.customize_plot
-            )
-        self.add_menu_action(self.window_menu, self.customize_action)
-
         self.preferences_action=QtWidgets.QAction("Edit Preferences",
             self,
-            shortcut="Ctrl+Alt+E",
+            shortcut=QtGui.QKeySequence("Ctrl+Alt+E"),
             triggered=self.edit_preferences
             )
 #        self.add_menu_action(self.window_menu, self.preferences_action)
@@ -1082,7 +1105,7 @@ class MainWindow(QtWidgets.QMainWindow):
                                      "JPEG/PNG Files (*.jpg *.jpeg *.png)"))
             fname = getOpenFileName(self, 'Open Image File',
                                     self.default_directory, file_filter)
-            if fname is None:
+            if fname is None or not os.path.exists(fname):
                 return
             data = load_image(fname)
             if 'images' not in self.tree:
@@ -1103,30 +1126,21 @@ class MainWindow(QtWidgets.QMainWindow):
             directory = self.default_directory
             directory = QtWidgets.QFileDialog.getExistingDirectory(self, 
                         'Choose Directory', directory)
+            if directory is None or not os.path.exists(directory):
+                return
             tree_files = [self.tree[root].nxfilename for root in self.tree]
             nxfiles = sorted([f for f in os.listdir(directory) 
                               if ((f.endswith('.nxs') or f.endswith('.nx5') or
                                    f.endswith('.h5') or f.endswith('hdf5') or
                                    f.endswith('hdf') or f.endswith('.cxi') or
                                    f.endswith('nxspe')) and
-                              os.path.join(directory,f) not in tree_files and
-                              not os.path.islink(os.path.join(directory,f)))],
+                              os.path.join(directory, f) not in tree_files and
+                              not os.path.islink(os.path.join(directory, f)))],
                              key=natural_sort)
             if len(nxfiles) == 0:
                 raise NeXusError("No NeXus files found in directory")
-            if confirm_action("Open %s NeXus files" % len(nxfiles),
-                              '\n'.join(nxfiles)):
-                for nxfile in nxfiles:
-                    fname = os.path.join(directory, nxfile)
-                    if is_file_locked(fname, wait=1):
-                        continue
-                    name = self.tree.get_name(fname)
-                    self.tree[name] = nxload(fname)
-                self.treeview.select_node(self.tree[name])
-                self.treeview.setFocus()
-                self.default_directory = os.path.dirname(fname)
-                logging.info(
-                    "%s NeXus files opened from %s" % (len(nxfiles), directory))
+            dialog = DirectoryDialog(nxfiles, directory)
+            dialog.show()
         except NeXusError as error:
             report_error("Opening Directory", error)
 
@@ -1305,7 +1319,7 @@ class MainWindow(QtWidgets.QMainWindow):
         try:
             node = self.treeview.get_node()
             if isinstance(node, NXdata):
-                if node.nxsignal.ndim == 1:
+                if node.ndim == 1:
                     dialog = ExportDialog(node, parent=self)
                     dialog.show()
                 else:
@@ -1459,6 +1473,10 @@ class MainWindow(QtWidgets.QMainWindow):
                 elif node.is_plottable():
                     dialog = PlotDialog(node, parent=self)
                     dialog.show()
+                elif (isinstance(node, NXfield) and 
+                      node.size == 1 and node.is_numeric()):
+                    dialog = PlotScalarDialog(node, parent=self)
+                    dialog.show()
                 else:
                     raise NeXusError("Data not plottable")
         except NeXusError as error:
@@ -1637,21 +1655,63 @@ class MainWindow(QtWidgets.QMainWindow):
         except NeXusError as error:
             report_error("Renaming Data", error)
 
+    def copy_node(self, node):
+        import tempfile
+        self._memroot = nxload(tempfile.mkstemp(suffix='.nxs')[1], mode='w',
+                               driver='core', backing_store=False)
+        self._memroot['entry'] = NXentry()
+        if isinstance(node, NXlink):
+            node = node.nxlink
+        self._memroot['entry'][node.nxname] = node
+        self._memroot['entry'].attrs['link'] = [node.nxname, node.nxpath, 
+                                                str(node.nxfilename)]
+        return self._memroot['entry'][node.nxname]
+
+    @property
+    def copied_link(self):
+        try:
+            return self._memroot['entry'].attrs['link']
+        except Exception:
+            return None
+
     def copy_data(self):
         try:
             node = self.treeview.get_node()
             if not isinstance(node, NXroot):
-                self.copied_node = self.treeview.get_node()
-                logging.info("'%s' copied" % self.copied_node.nxpath)
+                self.copied_node = self.copy_node(node)
+                logging.info("'%s' copied" % node.nxpath)
             else:
                 raise NeXusError("Use 'Duplicate File' to copy an NXroot group")
         except NeXusError as error:
             report_error("Copying Data", error)
 
+    def cut_data(self):
+        try:
+            node = self.treeview.get_node()
+            if isinstance(node, NXroot):
+                raise NeXusError("Cannot cut an NXroot group")
+            elif node.nxgroup.is_external():
+                raise NeXusError(
+                    "Cannot cut object in an externally linked group")
+            elif node.nxgroup.nxfilemode and node.nxgroup.nxfilemode == 'r':
+                raise NeXusError("NeXus file is locked")
+            else:
+                if confirm_action("Are you sure you want to cut '%s'?"
+                                  % (node.nxroot.nxname+node.nxpath)):
+                    self.copied_node = self.copy_node(node)
+                    logging.info("'%s' cut" % node.nxpath)
+                    del node.nxgroup[node.nxname]
+        except NeXusError as error:
+            report_error("Cutting Data", error)
+
     def paste_data(self):
         try:
             node = self.treeview.get_node()
-            if isinstance(node, NXgroup) and self.copied_node is not None:
+            if node.nxfilemode and node.nxfilemode == 'r':
+                raise NeXusError("NeXus file is locked")
+            elif isinstance(node, NXgroup) and self.copied_node is not None:
+                if self.copied_node.nxname in node:
+                    self.copied_node.nxname = self.copied_node.nxname + '_copy'
                 if node.nxfilemode != 'r':
                     node.insert(self.copied_node)
                     logging.info("'%s' pasted to '%s'"
@@ -1664,9 +1724,15 @@ class MainWindow(QtWidgets.QMainWindow):
     def paste_link(self):
         try:
             node = self.treeview.get_node()
-            if isinstance(node, NXgroup) and self.copied_node is not None:
+            if isinstance(node, NXgroup) and self.copied_link is not None:
                 if node.nxfilemode != 'r':
-                    node.makelink(self.copied_node)
+                    _name, _target, _filename = self.copied_link
+                    if _name in node:
+                        _name = _name + '_copy'
+                    if _filename == 'None' or node.nxfilename == _filename:
+                        node[_name] = NXlink(_target)
+                    else:
+                        node[_name] = NXlink(_target, _filename)
                     logging.info("'%s' pasted as link to '%s'"
                                  % (self.copied_node.nxpath, node.nxpath))
                 else:
@@ -1677,18 +1743,17 @@ class MainWindow(QtWidgets.QMainWindow):
     def delete_data(self):
         try:
             node = self.treeview.get_node()
-            if node is not None:
-                if node.nxfilemode != 'r':
-                    if confirm_action("Are you sure you want to delete '%s'?"
-                                      % (node.nxroot.nxname+node.nxpath)):
-                        del node.nxgroup[node.nxname]
-                        logging.info("'%s' deleted" % 
-                                     (node.nxroot.nxname+node.nxpath))
-                elif node.is_external():
-                    raise NeXusError(
-                        "Cannot delete object in an externally linked group")
-                else:
-                    raise NeXusError("NeXus file is locked")
+            if isinstance(node, NXroot) and node.nxfilemode:
+                raise NeXusError("Cannot delete a NeXus file")
+            elif node.nxgroup.is_external():
+                raise NeXusError(
+                    "Cannot delete object in an externally linked group")
+            elif node.nxgroup.nxfilemode and node.nxgroup.nxfilemode == 'r':
+                raise NeXusError("NeXus file is locked")
+            elif confirm_action("Are you sure you want to delete '%s'?"
+                                % (node.nxroot.nxname+node.nxpath)):
+                del node.nxgroup[node.nxname]
+                logging.info("'%s' deleted" % (node.nxroot.nxname+node.nxpath))
         except NeXusError as error:
             report_error("Deleting Data", error)
 
@@ -1739,13 +1804,10 @@ class MainWindow(QtWidgets.QMainWindow):
                 if node.nxfilemode != 'r':
                     if node.nxgroup is None:
                         raise NeXusError("There is no parent group")
-                    if 'default' in node.nxgroup.attrs:
+                    elif node.nxgroup.get_default():
                         if not confirm_action("Override existing default?"):
                             return
-                    node.nxgroup.attrs['default'] = node.nxname
-                    if node.nxgroup in node.nxroot.values():
-                        if 'default' not in node.nxroot.attrs:
-                            node.nxroot.attrs['default'] = node.nxgroup.nxname
+                    node.set_default()
                     logging.info("Default set to '%s'" % node.nxpath)
                 else:
                     raise NeXusError("NeXus file is locked")
@@ -1954,6 +2016,13 @@ class MainWindow(QtWidgets.QMainWindow):
                 triggered=lambda: self.plotviews[label].make_active(),
                 checkable=False)
             self.window_menu.addAction(self.active_action[number])
+        elif label == 'Scan':
+            self.active_action[number] = QtWidgets.QAction(label,
+                self,
+                shortcut=QtGui.QKeySequence("Ctrl+Shift+Alt+S"),
+                triggered=lambda: self.plotviews[label].make_active(),
+                checkable=False)
+            self.window_menu.addAction(self.active_action[number])
         elif label == 'Fit':
             self.active_action[number] = QtWidgets.QAction(label,
                 self,
@@ -2007,28 +2076,12 @@ class MainWindow(QtWidgets.QMainWindow):
             self.update_active(number)
             self.plotviews[self.active_action[number].text()].make_active()
 
-    def limit_axes(self):
-        try:
-            if 'limit' not in self.panels:
-                self.panels['limit'] = LimitDialog(parent=self)
-            self.panels['limit'].activate(self.plotview.label)
-        except NeXusError as error:
-            report_error("Changing Plot Limits", error)
-
     def reset_axes(self):
         try:
             self.plotview.reset_plot_limits()
         except NeXusError as error:
             report_error("Resetting Plot Limits", error)
 
-    def customize_plot(self):
-        try:
-            if 'customize' not in self.panels:
-                self.panels['customize'] = CustomizeDialog(parent=self)
-            self.panels['customize'].activate(self.plotview.label)
-        except NeXusError as error:
-            report_error("Customizing Plot", error)
-
     def edit_preferences(self):
         try:
             dialog = PreferencesDialog(parent=self)
@@ -2057,16 +2110,49 @@ class MainWindow(QtWidgets.QMainWindow):
         except NeXusError as error:
             report_error("Showing Log File", error)
 
+    def show_customize_panel(self):
+        try:
+            if 'customize' not in self.panels:
+                self.panels['customize'] = CustomizeDialog(parent=self)
+            self.panels['customize'].activate(self.active_plotview.label)
+        except NeXusError as error:
+            report_error("Showing Customize Panel", error)
+
+    def show_limits_panel(self):
+        try:
+            if 'limit' not in self.panels:
+                self.panels['limit'] = LimitDialog(parent=self)
+            self.panels['limit'].activate(self.active_plotview.label)
+        except NeXusError as error:
+            report_error("Showing Limits Panel", error)
+
     def show_projection_panel(self):
-        if self.plotview.label == 'Projection' or self.plotview.ndim == 1:
+        if self.active_plotview.label == 'Projection' or self.plotview.ndim == 1:
+            if ('projection' in self.panels and 
+                self.panels['projection'].isVisible()):
+                self.panels['projection'].raise_()
+                self.panels['projection'].activateWindow()
             return
         try:
             if 'projection' not in self.panels:
                 self.panels['projection'] = ProjectionDialog(parent=self)
-            self.panels['projection'].activate(self.plotview.label)
+            self.panels['projection'].activate(self.active_plotview.label)
         except NeXusError as error:
             report_error("Showing Projection Panel", error)
 
+    def show_scan_panel(self):
+        if self.plotview.label == 'Projection':
+            if 'scan' in self.panels and self.panels['scan'].isVisible():
+                self.panels['scan'].raise_()
+                self.panels['scan'].activateWindow()
+            return
+        try:
+            if 'scan' not in self.panels:
+                self.panels['scan'] = ScanDialog(parent=self)
+            self.panels['scan'].activate(self.plotview.label)
+        except NeXusError as error:
+            report_error("Showing Scan Panel", error)
+
     def show_script_window(self):
         if self.editors.tabs.count() == 0:
             self.new_script()
